import { Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { PipelineTask } from './pipeline-task.entity';
import { Repository } from 'typeorm';
import { CreatePipelineTaskInput } from './dtos/create-pipeline-task.input';
import { Pipeline } from '../pipelines/pipeline.entity';
import debug from 'debug';
import { AmqpConnection } from '@golevelup/nestjs-rabbitmq';

const log = debug('fennec:pipeline-tasks:service');

@Injectable()
export class PipelineTasksService {
  constructor(
    @InjectRepository(PipelineTask)
    private readonly repository: Repository<PipelineTask>,
    @InjectRepository(Pipeline)
    private readonly pipelineRepository: Repository<Pipeline>,
    private readonly amqpConnection: AmqpConnection,
  ) {}
  async addTask(dto: CreatePipelineTaskInput) {
    const pipeline = await this.pipelineRepository.findOneOrFail({
      where: { id: dto.pipelineId },
      relations: ['project'],
    });
    // const hasUnfinishedTask = await this.repository
    //   .findOne({
    //     pipelineId: dto.pipelineId,
    //     commit: dto.commit,
    //     status: In([TaskStatuses.pending, TaskStatuses.working]),
    //   })
    //   .then((val) => !isNil(val));
    // if (hasUnfinishedTask) {
    //   throw new ConflictException(
    //     'There are the same tasks among the unfinished tasks!',
    //   );
    // }
    const task = await this.repository.save(this.repository.create(dto));
    task.pipeline = pipeline;

    this.amqpConnection.publish('new-pipeline-task', 'mac', task);
    return task;
  }

  async findTaskById(id: string) {
    return await this.repository.findOneOrFail({ id });
  }

  async listTasksByPipelineId(pipelineId: string) {
    return await this.repository.find({ pipelineId });
  }

  async listTasksByCommitHash(hash: string) {
    return await this.repository.find({
      where: { commit: hash },
      order: { createdAt: 'DESC' },
    });
  }

  getRedisTokens(pipeline: Pipeline): [string, string] {
    return [`pipeline-${pipeline.id}:lck`, `pipeline-${pipeline.id}:tasks`];
  }
}
